import codecs

from Crypto.Cipher import AES
from Crypto.Protocol.KDF import HKDF
from Crypto.Hash import SHA256
from coincurve import PrivateKey, PublicKey
from coincurve.utils import get_valid_secret

AES_KEY_BYTES_LEN = 32
AES_CIPHER_MODE = AES.MODE_GCM

def aes_encrypt(key: bytes, plain_text: bytes):
    aes_cipher = AES.new(key, AES_CIPHER_MODE)

    encrypted, tag = aes_cipher.encrypt_and_digest(plain_text)
    cipher_text = bytearray()
    cipher_text.extend(aes_cipher.nonce)
    cipher_text.extend(tag)
    cipher_text.extend(encrypted)
    return bytes(cipher_text)

def aes_decrypt(key: bytes, cipher_text: bytes) -> bytes:
    iv = cipher_text[:16]
    tag = cipher_text[16:32]
    ciphered_data = cipher_text[32:]

    aes_cipher = AES.new(key, AES_CIPHER_MODE, nonce=iv)
    return aes_cipher.decrypt_and_verify(ciphered_data, tag)


def encapsulate(private_key: PrivateKey, peer_public_key: PublicKey) -> bytes:
    shared_point = peer_public_key.multiply(private_key.secret)
    master = private_key.public_key.format(compressed=False) + shared_point.format(
        compressed=False
    )
    derived = HKDF(master, AES_KEY_BYTES_LEN, b"", SHA256)
    return derived


def decapsulate(public_key: PublicKey, peer_private_key: PrivateKey) -> bytes:
    shared_point = public_key.multiply(peer_private_key.secret)
    master = public_key.format(compressed=False) + shared_point.format(compressed=False)
    derived = HKDF(master, AES_KEY_BYTES_LEN, b"", SHA256)
    return derived


def generate_key() -> PrivateKey:
    return PrivateKey(get_valid_secret())


def hex2pub(pub_hex: str) -> PublicKey:

    if pub_hex.startswith("0x") or pub_hex.startswith("0X"):
        return pub_hex[2:]

    uncompressed = codecs.decode(pub_hex, "hex")
    if len(uncompressed) == 64:
        uncompressed = b"\x04" + uncompressed

    return PublicKey(uncompressed)


def hex2prv(prv_hex: str) -> PrivateKey:

    if prv_hex.startswith("0x") or prv_hex.startswith("0X"):
        return prv_hex[2:]
    prv_hex = codecs.decode(prv_hex, 'hex')

    return PrivateKey(prv_hex)


import os

from coincurve.keys import PrivateKey
from .utils import generate_key

from . import encrypt

def main():
    lock = '.private_key'

    if os.path.exists(lock):
        with open(lock, 'rb') as pk:
            private_key = PrivateKey(pk.read())
            print('public key :', private_key.public_key.format(True).hex())
            pk.close()


    else:
        with open(lock, 'wb+') as pk:
            private_key = generate_key()
            pk.write(private_key.secret)
            print('public key :', private_key.public_key.format(True).hex())
            pk.close()


from coincurve.keys import PublicKey

from .utils import aes_decrypt, aes_encrypt, decapsulate, encapsulate, generate_key, hex2prv, hex2pub

def encrypt(receiver_pk: str, msg: bytes):

    ephemeral_key = generate_key()
    if isinstance(receiver_pk, str):
        receiver_pubkey = hex2pub(receiver_pk)
    else:
        raise TypeError("Invalid public key type")

    aes_key = encapsulate(ephemeral_key, receiver_pubkey)
    cipher_text = aes_encrypt(aes_key, msg)
    result = ephemeral_key.public_key.format(False) + cipher_text
    return result.hex()


def decrypt(receiver_sk: str, msg: bytes) -> bytes:

    if isinstance(receiver_sk, str):
        private_key = hex2prv(receiver_sk)
    else:
        raise TypeError("Invalid secret key type")

    pubkey = msg[0:65]
    encrypted = msg[65:]
    ephemeral_public_key = PublicKey(pubkey)

    aes_key = decapsulate(ephemeral_public_key, private_key)
    return aes_decrypt(aes_key, encrypted)


def private_key(loc = '.private_key') -> bytes:

    with open(loc, 'rb') as sk:
        private_key = sk.read()
        return private_key


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ecceran']

package_data = \
{'': ['*']}

install_requires = \
['coincurve==16.0.0', 'pycryptodome==3.12.0']

entry_points = \
{'console_scripts': ['ecceran = ecceran.__main__:main']}

setup_kwargs = {
    'name': 'ecceran',
    'version': '0.1.1',
    'description': 'Simple ECC encrypt decrypt',
    'long_description': "# Ecceran\n\nsimple ECC encrypt & decrypt\n\n### Install\n\n`$ pip install ecceran`\n\n### Generate Private Key\n\nrun `ecceran` in your command line for generate new secret key in `.private_key`\n\n```\n\n$ ecceran\n\npublic key : 02a4610d81d6c522ae67c2570********\n\n```\nthe public key will be used to decrypt. `generate` command will return public key from file `.private_key`, to re-generate need to delete file `private_key`\n\n### Get Private Key\n\n```python\n\nfrom ecceran import private_key\n\n\n\nprivate_key_app = private_key()\n\n# 5xa46gg0d81d6c522ae9ku2570********\n\n  \n\n```\n\n### Encrypt & Decrypt\n\n```python\n\nfrom ecceran import encrypt, decrypt, private_key\n\n\n\npubkey = '02a4610d81d6c522ae67c2570********'\n\nenc_data = encrypt(pubkey, b'test')\n\nprint(enc_data)\n\n# d2zx4d81d6xxx2va67c24470********\n\n  \n\nprikey = private_key()\n\nbyte = bytes.fromhex(enc_data)\n\ndec_data = decrypt(prikey, byte)\n\nprint(dec_data)\n\n# b'test'\n\n```",
    'author': 'hfrada',
    'author_email': 'madefrada@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
